require 'beanstalk-client'

class BeanStalk::Worker
  attr_accessor :config, :log, :connection, :stats
  
  def initialize(config = {})
    @config = BeanStalk::Worker::Config
    if File.exists? @config[:config_file]
      if config[:environment]
        @config.from_file @config[:config_file],
          :environment => config[:environment]
      else
        @config.from_file @config[:config_file]
      end
    end
    @config.merge!(config || {})
    
    @stats = {
      'received' => 0
    }
  end
  
  def beanstalk
    if @connection.nil?
      @connection = Beanstalk::Pool.new([
        BeanStalk::Worker::Config.beanstalk_uri
      ])
      @log.info("Connected to beanstalk")
    end
    @connection
  rescue
    @log.error("Could not connect to beanstalk.")
    reconnect
  end
  
  def initialize_logger
    @log = Logger.new(@config.log_location)
    @log.level = Logger.const_get(@config.log_level.upcase)
    @log.error("Logging started")
  end
  
  def initialize_beanstalk    
    beanstalk.watch(@config[:beanstalk][:tube])
    beanstalk.use(@config[:beanstalk][:tube])
    beanstalk.ignore('default')
  rescue
    @log.error("Could not connect to beanstalk.")
    reconnect
  end
  
  def reconnect
    @connection = nil
    @log.error("Sleeping 30 seconds")
    sleep(30)
    @log.error("Attempting to reconnect")
    start
  end
  
  def start(received = -1)
    initialize_logger
    initialize_beanstalk
    while (received == -1) || (@stats['received'] < received)
      begin
        job = beanstalk.reserve
        
        @log.debug("job #{job.inspect}")
        @log.debug("job #{job.body.inspect}")
        
        @stats['received'] += 1
        
        job.delete if work(job)
      rescue Beanstalk::NotConnected => e
        @log.error("Beanstalk disconnected")
        reconnect
      rescue Exception => e
        @log.error("Caught exception #{e.to_s}")
        exit
      end
    end
  end
  
  def work(job)
    return true
  end
end
